@@ -19,9 +19,6 @@
  * with this program; if not, write to the Free Software Foundation, Inc., 
  * 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
  */
-#include <stdlib.h>
-#include <stdio.h>
-#include <ctype.h>
 #include <zebra.h>
 
 #include "log.h"
