@@ -23,7 +23,18 @@
 #ifndef _ZEBRA_NETWORK_H
 #define _ZEBRA_NETWORK_H
 
-int readn (int, u_char *, int);
-int writen (int, const u_char *, int);
+/* Both readn and writen are deprecated and will be removed.  They are not
+   suitable for use with non-blocking file descriptors.
+ */
+extern int readn (int, u_char *, int);
+extern int writen (int, const u_char *, int);
+
+/* Set the file descriptor to use non-blocking I/O.  Returns 0 for success,
+   -1 on error. */
+extern int set_nonblocking(int fd);
+
+/* Does the I/O error indicate that the operation should be retried later? */
+#define ERRNO_IO_RETRY(EN) \
+	(((EN) == EAGAIN) || ((EN) == EWOULDBLOCK) || ((EN) == EINTR))
 
 #endif /* _ZEBRA_NETWORK_H */
