@@ -189,11 +189,13 @@ ospf_apiserver_term (void)
    * Free all client instances.  ospf_apiserver_free removes the node
    * from the list, so we examine the head of the list anew each time.
    */
-  while ( (apiserv = listgetdata (listhead (apiserver_list))) != NULL)
+  while ( apiserver_list &&
+         (apiserv = listgetdata (listhead (apiserver_list))) != NULL)
     ospf_apiserver_free (apiserv);
 
   /* Free client list itself */
-  list_delete (apiserver_list);
+  if (apiserver_list)
+    list_delete (apiserver_list);
 
   /* Free wildcard list */
   /* XXX  */
@@ -660,7 +662,7 @@ ospf_apiserver_accept (struct thread *thread)
   struct ospf_apiserver *apiserv;
   struct sockaddr_in peer_async;
   struct sockaddr_in peer_sync;
-  int peerlen;
+  unsigned int peerlen;
   int ret;
 
   /* THREAD_ARG (thread) is NULL */
@@ -767,7 +769,7 @@ ospf_apiserver_accept (struct thread *thread)
  * -----------------------------------------------------------
  */
 
-int
+static int
 ospf_apiserver_send_msg (struct ospf_apiserver *apiserv, struct msg *msg)
 {
   struct msg_fifo *fifo;
@@ -992,7 +994,7 @@ ospf_apiserver_unregister_opaque_type (struct ospf_apiserver *apiserv,
 }
 
 
-int
+static int
 apiserver_is_opaque_type_registered (struct ospf_apiserver *apiserv,
 				     u_char lsa_type, u_char opaque_type)
 {
@@ -1174,8 +1176,8 @@ ospf_apiserver_notify_ready_type11 (struct ospf_apiserver *apiserv)
   for (ALL_LIST_ELEMENTS (apiserv->opaque_types, node, nnode, r))
     {
       struct msg *msg;
-      struct in_addr noarea_id = { 0L };
-
+      struct in_addr noarea_id = { .s_addr = 0L };
+      
       if (r->lsa_type == OSPF_OPAQUE_AS_LSA)
 	{
 	  /* Yes, this opaque type is ready */
@@ -1268,7 +1270,7 @@ ospf_apiserver_handle_register_event (struct ospf_apiserver *apiserv,
  * -----------------------------------------------------------
  */
 
-int
+static int
 apiserver_sync_callback (struct ospf_lsa *lsa, void *p_arg, int int_arg)
 {
   struct ospf_apiserver *apiserv;
@@ -1296,11 +1298,11 @@ apiserver_sync_callback (struct ospf_lsa *lsa, void *p_arg, int int_arg)
     {
 
       /* Default area for AS-External and Opaque11 LSAs */
-      struct in_addr area_id = { 0L };
+      struct in_addr area_id = { .s_addr = 0L };
 
       /* Default interface for non Opaque9 LSAs */
-      struct in_addr ifaddr = { 0L };
-
+      struct in_addr ifaddr = { .s_addr = 0L };
+      
       if (lsa->area)
 	{
 	  area_id = lsa->area->area_id;
@@ -1518,7 +1520,7 @@ ospf_apiserver_opaque_lsa_new (struct ospf_area *area,
   if ((new->data = ospf_lsa_data_new (length)) == NULL)
     {
       zlog_warn ("ospf_apiserver_opaque_lsa_new: ospf_lsa_data_new() ?");
-      ospf_lsa_unlock (new);
+      ospf_lsa_unlock (&new);
       stream_free (s);
       return NULL;
     }
@@ -1883,7 +1885,7 @@ ospf_apiserver_lsa_refresher (struct ospf_lsa *lsa)
   if (ospf_lsa_install (ospf, new->oi, new) == NULL)
     {
       zlog_warn ("ospf_apiserver_lsa_refresher: ospf_lsa_install failed");
-      ospf_lsa_unlock (new);
+      ospf_lsa_unlock (&new);
       goto out;
     }
 
@@ -1997,7 +1999,7 @@ out:
 }
 
 /* Flush self-originated opaque LSA */
-int
+static int
 apiserver_flush_opaque_type_callback (struct ospf_lsa *lsa,
 				      void *p_arg, int int_arg)
 {
@@ -2356,11 +2358,11 @@ ospf_apiserver_clients_notify_ready_type11 (struct ospf *top)
 {
   struct listnode *node, *nnode;
   struct msg *msg;
-  struct in_addr id_null = { 0L };
+  struct in_addr id_null = { .s_addr = 0L };
   struct ospf_apiserver *apiserv;
-
+  
   assert (top);
-
+  
   if (!ospf_apiserver_is_ready_type11 (top))
     {
       zlog_warn ("AS not ready for type 11?");
@@ -2429,12 +2431,12 @@ void
 ospf_apiserver_clients_notify_ism_change (struct ospf_interface *oi)
 {
   struct msg *msg;
-  struct in_addr ifaddr = { 0L };
-  struct in_addr area_id = { 0L };
-
+  struct in_addr ifaddr = { .s_addr = 0L };
+  struct in_addr area_id = { .s_addr = 0L };
+  
   assert (oi);
   assert (oi->ifp);
-
+  
   if (oi->address)
     {
       ifaddr = oi->address->u.prefix4;
@@ -2444,7 +2446,7 @@ ospf_apiserver_clients_notify_ism_change (struct ospf_interface *oi)
       area_id = oi->area->area_id;
     }
 
-  msg = new_msg_ism_change (0, ifaddr, area_id, oi->ifp->status);
+  msg = new_msg_ism_change (0, ifaddr, area_id, oi->state);
   if (!msg)
     {
       zlog_warn ("apiserver_clients_notify_ism_change: msg_new failed");
@@ -2459,8 +2461,8 @@ void
 ospf_apiserver_clients_notify_nsm_change (struct ospf_neighbor *nbr)
 {
   struct msg *msg;
-  struct in_addr ifaddr = { 0L };
-  struct in_addr nbraddr = { 0L };
+  struct in_addr ifaddr = { .s_addr = 0L };
+  struct in_addr nbraddr = { .s_addr = 0L };
 
   assert (nbr);
 
@@ -2482,7 +2484,7 @@ ospf_apiserver_clients_notify_nsm_change (struct ospf_neighbor *nbr)
   msg_free (msg);
 }
 
-void
+static void
 apiserver_clients_lsa_change_notify (u_char msgtype, struct ospf_lsa *lsa)
 {
   struct msg *msg;
@@ -2490,10 +2492,10 @@ apiserver_clients_lsa_change_notify (u_char msgtype, struct ospf_lsa *lsa)
   struct ospf_apiserver *apiserv;
 
   /* Default area for AS-External and Opaque11 LSAs */
-  struct in_addr area_id = { 0L };
+  struct in_addr area_id = { .s_addr = 0L };
 
   /* Default interface for non Opaque9 LSAs */
-  struct in_addr ifaddr = { 0L };
+  struct in_addr ifaddr = { .s_addr = 0L };
 
   if (lsa->area)
     {
@@ -2583,15 +2585,15 @@ apiserver_clients_lsa_change_notify (u_char msgtype, struct ospf_lsa *lsa)
  */
 
 
-int
+static int
 apiserver_notify_clients_lsa (u_char msgtype, struct ospf_lsa *lsa)
 {
   struct msg *msg;
   /* default area for AS-External and Opaque11 LSAs */
-  struct in_addr area_id = { 0L };
+  struct in_addr area_id = { .s_addr = 0L };
 
   /* default interface for non Opaque9 LSAs */
-  struct in_addr ifaddr = { 0L };
+  struct in_addr ifaddr = { .s_addr = 0L };
 
   /* Only notify this update if the LSA's age is smaller than
      MAXAGE. Otherwise clients would see LSA updates with max age just
