@@ -46,8 +46,6 @@
 #include "ospfd/ospf_zebra.h"
 #include "ospfd/ospf_dump.h"
 
-#define DEBUG
-
 struct ospf_route *
 ospf_find_asbr_route (struct ospf *ospf,
 		      struct route_table *rtrs, struct prefix_ipv4 *asbr)
@@ -71,12 +69,11 @@ ospf_find_asbr_route (struct ospf *ospf,
 
   /* First try to find intra-area non-bb paths. */
   if (!CHECK_FLAG (ospf->config, OSPF_RFC1583_COMPATIBLE))
-    for (node = listhead ((struct list *) rn->info); node; nextnode (node))
-      if ((or = getdata (node)) != NULL)
-	if (or->cost < OSPF_LS_INFINITY)
-	  if (!OSPF_IS_AREA_ID_BACKBONE (or->u.std.area_id) &&
-	      or->path_type == OSPF_PATH_INTRA_AREA)
-	    listnode_add (chosen, or);
+    for (ALL_LIST_ELEMENTS_RO ((struct list *) rn->info, node, or))
+      if (or->cost < OSPF_LS_INFINITY)
+        if (!OSPF_IS_AREA_ID_BACKBONE (or->u.std.area_id) &&
+            or->path_type == OSPF_PATH_INTRA_AREA)
+          listnode_add (chosen, or);
 
   /* If none is found -- look through all. */
   if (listcount (chosen) == 0)
@@ -86,19 +83,18 @@ ospf_find_asbr_route (struct ospf *ospf,
     }
 
   /* Now find the route with least cost. */
-  for (node = listhead (chosen); node; nextnode (node))
-    if ((or = getdata (node)) != NULL)
-      if (or->cost < OSPF_LS_INFINITY)
-	{
-	  if (best == NULL)
-	    best = or;
-	  else if (best->cost > or->cost)
-	    best = or;
-	  else if (best->cost == or->cost &&
-		   IPV4_ADDR_CMP (&best->u.std.area_id,
-				  &or->u.std.area_id) < 0)
-	    best = or;
-	}
+  for (ALL_LIST_ELEMENTS_RO (chosen, node, or))
+    if (or->cost < OSPF_LS_INFINITY)
+      {
+        if (best == NULL)
+          best = or;
+        else if (best->cost > or->cost)
+          best = or;
+        else if (best->cost == or->cost &&
+                 IPV4_ADDR_CMP (&best->u.std.area_id,
+                                &or->u.std.area_id) < 0)
+          best = or;
+      }
 
   if (chosen != rn->info)
     list_delete (chosen);
@@ -126,45 +122,42 @@ ospf_find_asbr_route_through_area (struct route_table *rtrs,
 
       route_unlock_node (rn);
 
-      for (node = listhead ((struct list *) rn->info); node; nextnode (node))
-	if ((or = getdata (node)) != NULL)
-	  if (IPV4_ADDR_SAME (&or->u.std.area_id, &area->area_id))
-	    return or;
+      for (ALL_LIST_ELEMENTS_RO ((struct list *) rn->info, node, or))
+        if (IPV4_ADDR_SAME (&or->u.std.area_id, &area->area_id))
+          return or;
     }
 
   return NULL;
 }
 
-void
+static void
 ospf_ase_complete_direct_routes (struct ospf_route *ro, struct in_addr nexthop)
 {
   struct listnode *node;
   struct ospf_path *op;
 
-  for (node = listhead (ro->paths); node; nextnode (node))
-    if ((op = getdata (node)) != NULL)
-      if (op->nexthop.s_addr == 0)
-	op->nexthop.s_addr = nexthop.s_addr;
+  for (ALL_LIST_ELEMENTS_RO (ro->paths, node, op))
+    if (op->nexthop.s_addr == 0)
+      op->nexthop.s_addr = nexthop.s_addr;
 }
 
-int
+static int
 ospf_ase_forward_address_check (struct ospf *ospf, struct in_addr fwd_addr)
 {
   struct listnode *ifn;
   struct ospf_interface *oi;
 
-  for (ifn = listhead (ospf->oiflist); ifn; nextnode (ifn))
-    if ((oi = getdata (ifn)) != NULL)
-      if (if_is_operative (oi->ifp))
-	if (oi->type != OSPF_IFTYPE_VIRTUALLINK)
-	  if (IPV4_ADDR_SAME (&oi->address->u.prefix4, &fwd_addr))
-	    return 0;
+  for (ALL_LIST_ELEMENTS_RO (ospf->oiflist, ifn, oi))
+    if (if_is_operative (oi->ifp))
+      if (oi->type != OSPF_IFTYPE_VIRTUALLINK)
+        if (IPV4_ADDR_SAME (&oi->address->u.prefix4, &fwd_addr))
+          return 0;
   
   return 1;
 }
 
 /* Calculate ASBR route. */
-struct ospf_route *
+static struct ospf_route *
 ospf_ase_calculate_asbr_route (struct ospf *ospf,
 			       struct route_table *rt_network,
 			       struct route_table *rt_router,
@@ -184,31 +177,36 @@ ospf_ase_calculate_asbr_route (struct ospf *ospf,
 
   if (asbr_route == NULL)
     {
-      zlog_info ("ospf_ase_calculate(): Route to ASBR %s not found",
-		 inet_ntoa (asbr.prefix));
+      if (IS_DEBUG_OSPF (lsa, LSA))
+	zlog_debug ("ospf_ase_calculate(): Route to ASBR %s not found",
+		    inet_ntoa (asbr.prefix));
       return NULL;
     }
 
   if (!(asbr_route->u.std.flags & ROUTER_LSA_EXTERNAL))
     {
-      zlog_info ("ospf_ase_calculate(): Originating router is not an ASBR");
+      if (IS_DEBUG_OSPF (lsa, LSA))
+	zlog_debug ("ospf_ase_calculate(): Originating router is not an ASBR");
       return NULL;
     }
    
   if (al->e[0].fwd_addr.s_addr != 0)
     {
-      zlog_info ("ospf_ase_calculate(): "
-		 "Forwarding address is not 0.0.0.0.");
+      if (IS_DEBUG_OSPF (lsa, LSA))
+	zlog_debug ("ospf_ase_calculate(): "
+		    "Forwarding address is not 0.0.0.0.");
 
       if (! ospf_ase_forward_address_check (ospf, al->e[0].fwd_addr))
 	{
-	  zlog_info ("ospf_ase_calculate(): "
-		     "Forwarding address is one of our addresses, Ignore.");
+	  if (IS_DEBUG_OSPF (lsa, LSA))
+	    zlog_debug ("ospf_ase_calculate(): "
+			"Forwarding address is one of our addresses, Ignore.");
 	  return NULL;
         }
 
-      zlog_info ("ospf_ase_calculate(): "
-		 "Looking up in the Network Routing Table.");
+      if (IS_DEBUG_OSPF (lsa, LSA))
+	zlog_debug ("ospf_ase_calculate(): "
+		    "Looking up in the Network Routing Table.");
 
       /* Looking up the path to the fwd_addr from Network route. */
       asbr.family = AF_INET;
@@ -219,8 +217,9 @@ ospf_ase_calculate_asbr_route (struct ospf *ospf,
    
       if (rn == NULL)
 	{
-	  zlog_info ("ospf_ase_calculate(): "
-		     "Couldn't find a route to the forwarding address.");
+	  if (IS_DEBUG_OSPF (lsa, LSA))
+	    zlog_debug ("ospf_ase_calculate(): "
+			"Couldn't find a route to the forwarding address.");
 	  return NULL;
 	}
 
@@ -228,8 +227,9 @@ ospf_ase_calculate_asbr_route (struct ospf *ospf,
 
       if ((asbr_route = rn->info) == NULL)
 	{
-	  zlog_info ("ospf_ase_calculate(): "
-		     "Somehow OSPF route to ASBR is lost");
+	  if (IS_DEBUG_OSPF (lsa, LSA))
+	    zlog_debug ("ospf_ase_calculate(): "
+			"Somehow OSPF route to ASBR is lost");
 	  return NULL;
 	}
     }
@@ -237,7 +237,7 @@ ospf_ase_calculate_asbr_route (struct ospf *ospf,
   return asbr_route;
 }
 
-struct ospf_route *
+static struct ospf_route *
 ospf_ase_calculate_new_route (struct ospf_lsa *lsa,
 			      struct ospf_route *asbr_route, u_int32_t metric)
 {
@@ -255,13 +255,15 @@ ospf_ase_calculate_new_route (struct ospf_lsa *lsa,
 
   if (!IS_EXTERNAL_METRIC (al->e[0].tos))
     {
-      zlog_info ("Route[External]: type-1 created.");
+      if (IS_DEBUG_OSPF (lsa, LSA))
+	zlog_debug ("Route[External]: type-1 created.");
       new->path_type = OSPF_PATH_TYPE1_EXTERNAL;
       new->cost = asbr_route->cost + metric;		/* X + Y */
     }
   else
     {
-      zlog_info ("Route[External]: type-2 created.");
+      if (IS_DEBUG_OSPF (lsa, LSA))
+	zlog_debug ("Route[External]: type-2 created.");
       new->path_type = OSPF_PATH_TYPE2_EXTERNAL;
       new->cost = asbr_route->cost;			/* X */
       new->u.ext.type2_cost = metric;			/* Y */
@@ -305,18 +307,21 @@ ospf_ase_calculate_route (struct ospf *ospf, struct ospf_lsa * lsa)
       return 0;
     }
 
-  zlog_info ("Route[External]: Calculate AS-external-LSA to %s/%d",
-	     inet_ntoa (al->header.id), ip_masklen (al->mask));
+  if (IS_DEBUG_OSPF (lsa, LSA))
+    zlog_debug ("Route[External]: Calculate AS-external-LSA to %s/%d",
+		inet_ntoa (al->header.id), ip_masklen (al->mask));
   /* (1) If the cost specified by the LSA is LSInfinity, or if the
          LSA's LS age is equal to MaxAge, then examine the next LSA. */
   if ((metric = GET_METRIC (al->e[0].metric)) >= OSPF_LS_INFINITY)
     {
-      zlog_info ("Route[External]: Metric is OSPF_LS_INFINITY");
+      if (IS_DEBUG_OSPF (lsa, LSA))
+	zlog_debug ("Route[External]: Metric is OSPF_LS_INFINITY");
       return 0;
     }
   if (IS_LSA_MAXAGE (lsa))
     {
-      zlog_info ("Route[External]: AS-external-LSA is MAXAGE");
+      if (IS_DEBUG_OSPF (lsa, LSA))
+	zlog_debug ("Route[External]: AS-external-LSA is MAXAGE");
       return 0;
     }
   
@@ -324,7 +329,8 @@ ospf_ase_calculate_route (struct ospf *ospf, struct ospf_lsa * lsa)
      examine the next LSA. */
   if (IS_LSA_SELF (lsa))
     {
-      zlog_info ("Route[External]: AS-external-LSA is self originated");
+      if (IS_DEBUG_OSPF (lsa, LSA))
+	zlog_debug ("Route[External]: AS-external-LSA is self originated");
       return 0;
     }
 
@@ -345,12 +351,14 @@ ospf_ase_calculate_route (struct ospf *ospf, struct ospf_lsa * lsa)
   asbr_route = ospf_find_asbr_route (ospf, ospf->new_rtrs, &asbr);
   if (asbr_route == NULL)
     {
-      zlog_info ("Route[External]: Can't find originating ASBR route");
+      if (IS_DEBUG_OSPF (lsa, LSA))
+	zlog_debug ("Route[External]: Can't find originating ASBR route");
       return 0;
     }
   if (!(asbr_route->u.std.flags & ROUTER_LSA_EXTERNAL))
     {
-      zlog_info ("Route[External]: Originating router is not an ASBR");
+      if (IS_DEBUG_OSPF (lsa, LSA))
+	zlog_debug ("Route[External]: Originating router is not an ASBR");
       return 0;
     }
   
@@ -384,7 +392,9 @@ ospf_ase_calculate_route (struct ospf *ospf, struct ospf_lsa * lsa)
 	 consider the next in the list. */
       if (! ospf_ase_forward_address_check (ospf, al->e[0].fwd_addr))
 	{
-	  zlog_info ("Route[External]: Forwarding address is our router address");
+	  if (IS_DEBUG_OSPF (lsa, LSA))
+	    zlog_debug ("Route[External]: Forwarding address is our router "
+			"address");
 	  return 0;
 	}
       
@@ -396,7 +406,9 @@ ospf_ase_calculate_route (struct ospf *ospf, struct ospf_lsa * lsa)
       
       if (rn == NULL || (asbr_route = rn->info) == NULL)
 	{
-	  zlog_info ("Route[External]: Can't find route to forwarding address");
+	  if (IS_DEBUG_OSPF (lsa, LSA))
+	    zlog_debug ("Route[External]: Can't find route to forwarding "
+			"address");
 	  if (rn)
 	    route_unlock_node (rn);
 	  return 0;
@@ -451,8 +463,9 @@ ospf_ase_calculate_route (struct ospf *ospf, struct ospf_lsa * lsa)
 			       (struct prefix *) &p)) == NULL 
       || (or = rn->info) == NULL)
     {
-      zlog_info ("Route[External]: Adding a new route %s/%d",
-		 inet_ntoa (p.prefix), p.prefixlen);
+      if (IS_DEBUG_OSPF (lsa, LSA))
+	zlog_debug ("Route[External]: Adding a new route %s/%d",
+		    inet_ntoa (p.prefix), p.prefixlen);
 
       ospf_route_add (ospf->new_external_route, &p, new, asbr_route);
 
@@ -491,7 +504,8 @@ ospf_ase_calculate_route (struct ospf *ospf, struct ospf_lsa * lsa)
       /* New route is better */
       if (ret < 0)
 	{
-	  zlog_info ("Route[External]: New route is better");
+	  if (IS_DEBUG_OSPF (lsa, LSA))
+	    zlog_debug ("Route[External]: New route is better");
 	  ospf_route_subst (rn, new, asbr_route);
 	  if (al->e[0].fwd_addr.s_addr)
 	    ospf_ase_complete_direct_routes (new, al->e[0].fwd_addr);
@@ -501,13 +515,15 @@ ospf_ase_calculate_route (struct ospf *ospf, struct ospf_lsa * lsa)
       /* Old route is better */
       else if (ret > 0)
 	{
-	  zlog_info ("Route[External]: Old route is better");
+	  if (IS_DEBUG_OSPF (lsa, LSA))
+	    zlog_debug ("Route[External]: Old route is better");
 	  /* do nothing */
 	}
       /* Routes are equal */
       else
 	{
-	  zlog_info ("Route[External]: Routes are equal");
+	  if (IS_DEBUG_OSPF (lsa, LSA))
+	    zlog_debug ("Route[External]: Routes are equal");
 	  ospf_route_copy_nexthops (or, asbr_route->paths);
 	  if (al->e[0].fwd_addr.s_addr)
 	    ospf_ase_complete_direct_routes (or, al->e[0].fwd_addr);
@@ -522,7 +538,7 @@ ospf_ase_calculate_route (struct ospf *ospf, struct ospf_lsa * lsa)
   return 0;
 }
 
-int
+static int
 ospf_ase_route_match_same (struct route_table *rt, struct prefix *prefix,
 			   struct ospf_route *newor)
 {
@@ -567,10 +583,10 @@ ospf_ase_route_match_same (struct route_table *rt, struct prefix *prefix,
        
    /* Check each path. */
    for (n1 = listhead (or->paths), n2 = listhead (newor->paths);
-	n1 && n2; nextnode (n1), nextnode (n2))
+	n1 && n2; n1 = listnextnode (n1), n2 = listnextnode (n2))
      { 
-       op = getdata (n1);
-       newop = getdata (n2);
+       op = listgetdata (n1);
+       newop = listgetdata (n2);
        
        if (! IPV4_ADDR_SAME (&op->nexthop, &newop->nexthop))
 	 return 0;
@@ -578,7 +594,7 @@ ospf_ase_route_match_same (struct route_table *rt, struct prefix *prefix,
    return 1;
 }
 
-int
+static int
 ospf_ase_compare_tables (struct route_table *new_external_route,
 			 struct route_table *old_external_route)
 {
@@ -605,7 +621,7 @@ ospf_ase_compare_tables (struct route_table *new_external_route,
   return 0;
 }
 
-int
+static int
 ospf_ase_calculate_timer (struct thread *t)
 {
   struct ospf *ospf;
@@ -627,9 +643,8 @@ ospf_ase_calculate_timer (struct thread *t)
 
       /*  This version simple adds to the table all NSSA areas  */
       if (ospf->anyNSSA)
-	for (node = listhead (ospf->areas); node; nextnode (node))
+	for (ALL_LIST_ELEMENTS_RO (ospf->areas, node, area))
 	  {
-	    area = getdata (node);
 	    if (IS_DEBUG_OSPF_NSSA)
 	      zlog_debug ("ospf_ase_calculate_timer(): looking at area %s",
 			 inet_ntoa (area->area_id));
@@ -696,7 +711,7 @@ ospf_ase_register_external_lsa (struct ospf_lsa *lsa, struct ospf *top)
   /* We assume that if LSA is deleted from DB
      is is also deleted from this RT */
 
-  listnode_add (lst, ospf_lsa_lock (lsa));
+  listnode_add (lst, ospf_lsa_lock (lsa)); /* external_lsas lst */
 }
 
 void
@@ -715,18 +730,12 @@ ospf_ase_unregister_external_lsa (struct ospf_lsa *lsa, struct ospf *top)
 
   rn = route_node_get (top->external_lsas, (struct prefix *) &p);
   lst = rn->info;
-#ifdef ORIGINAL_CODING
-  assert (lst);
 
-  listnode_delete (lst, lsa);
-  ospf_lsa_unlock (lsa);
-#else /* ORIGINAL_CODING */
   /* XXX lst can be NULL */
   if (lst) {
     listnode_delete (lst, lsa);
-    ospf_lsa_unlock (lsa);
+    ospf_lsa_unlock (&lsa); /* external_lsas list */
   }
-#endif /* ORIGINAL_CODING */
 }
 
 void
@@ -735,17 +744,16 @@ ospf_ase_external_lsas_finish (struct route_table *rt)
   struct route_node *rn;
   struct ospf_lsa *lsa;
   struct list *lst;
-  struct listnode *node;
+  struct listnode *node, *nnode;
   
   for (rn = route_top (rt); rn; rn = route_next (rn))
     if ((lst = rn->info) != NULL)
       {
-	for (node = listhead (lst); node; node = nextnode (node))
-	  if ((lsa = getdata (node)) != NULL)
-	    ospf_lsa_unlock (lsa);
+	for (ALL_LIST_ELEMENTS (lst, node, nnode, lsa))
+          ospf_lsa_unlock (&lsa); /* external_lsas lst */
 	list_delete (lst);
       }
-  
+    
   route_table_finish (rt);
 }
 
@@ -785,9 +793,8 @@ ospf_ase_incremental_update (struct ospf *ospf, struct ospf_lsa *lsa)
   assert (rn && rn->info);
   lsas = rn->info;
   
-  for (node = listhead (lsas); node; nextnode (node))
-    if ((lsa = getdata (node)) != NULL)
-      ospf_ase_calculate_route (ospf, lsa);
+  for (ALL_LIST_ELEMENTS_RO (lsas, node, lsa))
+    ospf_ase_calculate_route (ospf, lsa);
 
   /* prepare temporary old routing table for compare */
   tmp_old = route_table_init ();
