@@ -201,6 +201,14 @@ struct zebra_if
   struct irdp_interface irdp;
 #endif
 
+#ifdef SUNOS_5
+  /* the real IFF_UP state of the primary interface.
+   * need this to differentiate between all interfaces being
+   * down (but primary still plumbed) and primary having gone
+   * ~IFF_UP, and all addresses gone.
+   */
+  u_char primary_state;
+#endif /* SUNOS_5 */
 };
 
 extern void if_delete_update (struct interface *ifp);
@@ -208,11 +216,12 @@ extern void if_add_update (struct interface *ifp);
 extern void if_up (struct interface *);
 extern void if_down (struct interface *);
 extern void if_refresh (struct interface *);
+extern void if_flags_update (struct interface *, uint64_t);
 extern int if_subnet_add (struct interface *, struct connected *);
 extern int if_subnet_delete (struct interface *, struct connected *);
 
 #ifdef HAVE_PROC_NET_DEV
-extern int ifstat_update_proc (void);
+extern void ifstat_update_proc (void);
 #endif /* HAVE_PROC_NET_DEV */
 #ifdef HAVE_NET_RT_IFLIST
 extern void ifstat_update_sysctl (void);
